@@ -2,7 +2,9 @@ import json
 import os
 import sys
 from collections import defaultdict
-from typing import Union, List, Any, Callable, Type, Dict, Set
+from typing import Union, List, Any, Callable, Dict, Set, Optional
+from enum import Enum
+import yaml
 
 from pytconf.color_utils import (
     print_highlight,
@@ -31,12 +33,22 @@ SPECIAL_FUNCTION_GROUP_SHOW = False
 DEFAULT_FUNCTION_DESCRIPTION = "No function description available"
 
 
+class ConfigType(Enum):
+    USER = 1
+    SYSTEM = 2
+
+
+class ConfigFormat(Enum):
+    JSON = 1
+    YAML = 2
+
+
 class MetaConfig(type):
     """
     Meta class for all configs
     """
-    def __new__(mcs, name, bases, namespace):
-        ret = super().__new__(mcs, name, bases, namespace)
+    def __new__(cls, name, bases, namespace):
+        ret = super().__new__(cls, name, bases, namespace)
         i = 0
         for k, v in namespace.items():
             if not k.startswith("__") and not isinstance(v, classmethod):
@@ -60,8 +72,8 @@ class PytconfConf:
     def __init__(self):
         self.main_function: Union[Callable, None] = None
         self.main_description: str = "No application description"
-        self.function_name_to_configs: Dict[str, List[Type[Config]]] = {}
-        self.function_name_to_suggest_configs: Dict[str, List[Type[Config]]] = {}
+        self.function_name_to_configs: Dict[str, List[Config]] = {}
+        self.function_name_to_suggest_configs: Dict[str, List[Config]] = {}
         self.function_name_to_callable: Dict[str, Callable] = {}
         self.allow_free_args: Dict[str, bool] = {}
         self.min_free_args: Dict[str, Union[int, None]] = {}
@@ -96,8 +108,8 @@ class PytconfConf:
         function: Callable,
         description: str,
         name: str,
-        configs: List[Type[Config]] = (),
-        suggest_configs: List[Type[Config]] = (),
+        configs: Optional[List[Config]],
+        suggest_configs: Optional[List[Config]],
         group: str = DEFAULT_FUNCTION_GROUP_NAME,
         allow_free_args: bool = False,
         min_free_args: Union[int, None] = None,
@@ -105,8 +117,14 @@ class PytconfConf:
     ):
         self.description[name] = description
         self.function_name_to_callable[name] = function
-        self.function_name_to_configs[name] = configs
-        self.function_name_to_suggest_configs[name] = suggest_configs
+        if configs is None:
+            self.function_name_to_configs[name] = []
+        else:
+            self.function_name_to_configs[name] = configs
+        if suggest_configs is None:
+            self.function_name_to_suggest_configs[name] = []
+        else:
+            self.function_name_to_suggest_configs[name] = suggest_configs
         self.function_group_names[group].add(name)
         self.allow_free_args[name] = allow_free_args
         self.min_free_args[name] = min_free_args
@@ -230,11 +248,14 @@ class PytconfConf:
             for k, v in new_flags.items():
                 flags[k] = v
 
-    def get_system_config(self):
-        return f"/etc/{self.app_name}.json"
-
-    def get_user_config(self):
-        return os.path.expanduser(f"~/.config/{self.app_name}.json")
+    def get_config(self, config_type: ConfigType, config_format: ConfigFormat):
+        if config_format == ConfigFormat.JSON:
+            suffix = "json"
+        if config_format == ConfigFormat.YAML:
+            suffix = "yaml"
+        if config_type == ConfigType.USER:
+            return os.path.expanduser(f"~/.config/{self.app_name}.{suffix}")
+        return f"/etc/{self.app_name}.{suffix}"
 
     def register_function_group(
         self,
@@ -293,8 +314,10 @@ class PytconfConf:
         self.free_args = []
 
         # read config files
-        self.read_flags_from_config(file_name=self.get_system_config(), flags=flags)
-        self.read_flags_from_config(file_name=self.get_user_config(), flags=flags)
+        self.read_flags_from_config(file_name=self.get_config(ConfigType.SYSTEM, ConfigFormat.JSON), flags=flags)
+        self.read_flags_from_config(file_name=self.get_config(ConfigType.SYSTEM, ConfigFormat.YAML), flags=flags)
+        self.read_flags_from_config(file_name=self.get_config(ConfigType.USER, ConfigFormat.JSON), flags=flags)
+        self.read_flags_from_config(file_name=self.get_config(ConfigType.USER, ConfigFormat.YAML), flags=flags)
 
         function_selected = self.get_function_selected(args, errors)
 
@@ -319,13 +342,11 @@ class PytconfConf:
                         errors.unset_show_errors()
                     else:
                         errors.add_error(f"free args are not allowed [{','.join(self.free_args)}]")
-
-        if function_selected is None:
+            self.process_flags(function_selected, flags, errors)
+        else:
             errors.add_error("no command is selected")
             errors.set_do_help()
             errors.unset_show_errors()
-        else:
-            self.process_flags(function_selected, flags, errors)
 
         if errors.have_errors() or errors.get_do_help():
             if errors.get_show_errors():
@@ -339,9 +360,16 @@ class PytconfConf:
                 sys.exit(1)
             return
 
+        self.launch(launch, function_selected, errors)
+
+    def launch(self, launch: bool, function_selected: Optional[str], errors):
         if launch:
-            function_to_run = self.function_name_to_callable[function_selected]
-            function_to_run()
+            if function_selected is None:
+                errors.add_error("no function to launch")
+                errors.set_do_help()
+            else:
+                function_to_run = self.function_name_to_callable[function_selected]
+                function_to_run()
 
     def parse_args(self, args, errors, flags):
         free_args_started = False
@@ -429,19 +457,20 @@ class PytconfConf:
                     html_gen.line("td", more_help)
 
     @classmethod
-    def write_config_file_json(cls, filename: str) -> None:
+    def write_config_file(cls, filename: str, config_format: ConfigFormat) -> None:
         values: Dict[str, str] = {}
         for config in the_registry.yield_configs():
             for name, param in the_registry.yield_name_data_for_config(config):
                 values[name] = param.t2s(param.default)
         with open(filename, "wt") as f:
-            json.dump(values, f, indent=4)
+            if config_format == ConfigFormat.JSON:
+                json.dump(values, f, indent=4)
+            if config_format == ConfigFormat.YAML:
+                yaml.dump(values, f, indent=4)
 
-    def write_config_file_json_user(self) -> None:
-        self.write_config_file_json(self.get_user_config())
-
-    def write_config_file_json_system(self) -> None:
-        self.write_config_file_json(self.get_system_config())
+    def write_config(self, config_type: ConfigType, config_format: ConfigFormat) -> None:
+        filename = self.get_config(config_type, config_format)
+        self.write_config_file(filename, config_format)
 
 
 _pytconf = PytconfConf()
@@ -504,8 +533,8 @@ def register_main(
 def register_endpoint(
     description: str,
     name: Union[str, None] = None,
-    configs: List[Type[Config]] = (),
-    suggest_configs: List[Type[Config]] = (),
+    configs: Optional[List[Config]] = None,
+    suggest_configs: Optional[List[Config]] = None,
     group: str = DEFAULT_FUNCTION_GROUP_NAME,
     allow_free_args: bool = False,
     min_free_args: Union[int, None] = None,
@@ -539,8 +568,8 @@ def register_function(
     function: Callable,
     description: str,
     name: str,
-    configs: List[Type[Config]] = (),
-    suggest_configs: List[Type[Config]] = (),
+    configs: Optional[List[Config]] = None,
+    suggest_configs: Optional[List[Config]] = None,
     group: str = DEFAULT_FUNCTION_GROUP_NAME,
     allow_free_args: bool = False,
     min_free_args: Union[int, None] = None,
@@ -559,25 +588,11 @@ def register_function(
     )
 
 
-def write_config_file_json_user() -> None:
-    filename = get_pytconf().get_user_config()
-    if not os.path.isfile(filename):
-        get_pytconf().write_config_file_json_user()
+def write_config_file(config_type: ConfigType, config_format: ConfigFormat) -> None:
+    get_pytconf().write_config(config_type, config_format)
 
 
-def rm_config_file_json_user() -> None:
-    filename = get_pytconf().get_user_config()
-    if os.path.isfile(filename):
-        os.unlink(filename)
-
-
-def write_config_file_json_system() -> None:
-    filename = get_pytconf().get_system_config()
-    if not os.path.isfile(filename):
-        get_pytconf().write_config_file_json_system()
-
-
-def rm_config_file_json_system() -> None:
-    filename = get_pytconf().get_system_config()
+def rm_config_file(config_type: ConfigType, config_format: ConfigFormat) -> None:
+    filename = get_pytconf().get_config(config_type, config_format)
     if os.path.isfile(filename):
         os.unlink(filename)
